//#!/usr/local/bin/tcc -run
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <getopt.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <string.h>
#include <errno.h>

#define DefaultMaxFD	(40960*4)
#define BUFLEN		1024
#define errExit(msg)     do { \
	perror(msg); exit(EXIT_FAILURE); \
} while (0)

/*
static struct option longopts[] = {
  { "help", 0, 0, 'h' },
  { "fd-to-expend", 0, 0, 'n' },
  { NULL, 0, 0, 0 }
};
*/

void usage(void);
/*----------------------------------------------------------------------------*/

int main(int argc, char **argv)
{
	int fd = 0, c, i;
	int nfd = 0;
	char *cmd = NULL;
	char cmdline[2048] = { 0 };
	int res;
	struct rlimit rlim;
	unsigned long max_open = DefaultMaxFD;

	//set the max open files
	memset(&rlim, 0, sizeof(rlim));
	if (getenv("maxOpen"))
		max_open=strtoul(getenv("maxOpen"),NULL,0);
	rlim.rlim_cur = max_open;
	rlim.rlim_max = max_open;
 	res = setrlimit(RLIMIT_NOFILE, &rlim);
	if (res)
		perror("change open file limit failed.\n");

	//debug: argv before getopt
	if (getenv("xdebug")) {
		for (i=0; i < argc; i++) printf("B-argv[%d] %s\n", i, argv[i]);
		printf("\n");
	}

	while ((c = getopt_long(argc, argv, "n:ht",
				NULL, NULL)) != -1) {
		switch (c) {
		case 'n':
			nfd=atoi(optarg);
			break;
		case 't':
			break;
		case 'h':
		default:
			goto out_usage;
		}
	}

	//debug: argv after getopt
	if (getenv("xdebug")) {
		for (i=0; i < argc; i++) printf("A-argv[%d] %s\n", i, argv[i]);
		printf("\n");
		for (c=optind; c < argc; c++) printf("P=argv[%d] %s\n", c, argv[c]);
		printf("\n");
	}

	if (optind > argc - 1)
		goto out_usage;

	//get open file limit
 	res = getrlimit(RLIMIT_NOFILE, &rlim);
	if (res) {
		perror("get open file limit failed.\n");
	} else {
		printf("==> now the open file limit: (%lu).(%lu)\n", 
			rlim.rlim_cur, rlim.rlim_max);
	}

	//trace arg value
	cmd = argv[optind];
	printf("==> fd to expend = [%d]\n", nfd);
	printf("==> cmd = [%s]\n", cmd);

	//expend the fd
	fd = open("/", O_RDONLY);
	printf("==> first open fd = [%d]\n", fd);
	for (i = 1; i < nfd; i++) {
		c = open("/", O_RDONLY);
		if (c < 0) {
			fprintf(stderr, "prefd = [%d] ", fd);
			errExit("open fail.");
		} else
			fd = c;
		/* yes the fd leak is intentional */
	}
	printf("==> now max fd = [%d]\n", fd);
	for (c=optind; c < argc; c++)
		sprintf(cmdline + strlen(cmdline), "%s ", argv[c]);
	printf("==> exec the cmdline [%s]\n", cmdline);
	res = execvp(argv[optind], &argv[optind]);

	exit(res);

out_usage:
	usage();
	exit(EXIT_FAILURE);
}

void usage()
{
	fprintf(stderr, "usage: [maxOpen=<num>] $prog [-h] [-n fsNum] <commond args>\n");
}

